@@ -35,11 +35,12 @@
 #include <ofono/modem.h>
 #include <ofono/gprs-context.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 #include "gatppp.h"
 
-#include "atmodem.h"
 #include "vendor.h"
 
 #define TUN_DEV "/dev/net/tun"
@@ -289,37 +290,35 @@ static void at_gprs_activate_primary(struct ofono_gprs_context *gc,
 
 	len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"", ctx->cid);
 
-	if (ctx->apn) {
-		switch (gcd->vendor) {
-		case OFONO_VENDOR_UBLOX:
-			/*
-			 * U-blox modems require a magic prefix to the APN to
-			 * specify the authentication method to use in the
-			 * network.  See UBX-13002752 - R21.
-			 *
-			 * As the response of the read command omits this magic
-			 * prefix, this is the least invasive place to set it.
-			 */
-			switch (ctx->auth_method) {
-			case OFONO_GPRS_AUTH_METHOD_CHAP:
-				snprintf(buf + len, sizeof(buf) - len - 3,
-						",\"CHAP:%s\"", ctx->apn);
-				break;
-			case OFONO_GPRS_AUTH_METHOD_PAP:
-				snprintf(buf + len, sizeof(buf) - len - 3,
-						",\"PAP:%s\"", ctx->apn);
-				break;
-			case OFONO_GPRS_AUTH_METHOD_NONE:
-				snprintf(buf + len, sizeof(buf) - len - 3,
-						",\"%s\"", ctx->apn);
-				break;
-			}
+	switch (gcd->vendor) {
+	case OFONO_VENDOR_UBLOX:
+		/*
+		 * U-blox modems require a magic prefix to the APN to
+		 * specify the authentication method to use in the
+		 * network.  See UBX-13002752 - R21.
+		 *
+		 * As the response of the read command omits this magic
+		 * prefix, this is the least invasive place to set it.
+		 */
+		switch (ctx->auth_method) {
+		case OFONO_GPRS_AUTH_METHOD_CHAP:
+			snprintf(buf + len, sizeof(buf) - len - 3,
+					",\"CHAP:%s\"", ctx->apn);
 			break;
-		default:
-			snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
-					ctx->apn);
+		case OFONO_GPRS_AUTH_METHOD_PAP:
+			snprintf(buf + len, sizeof(buf) - len - 3,
+					",\"PAP:%s\"", ctx->apn);
+			break;
+		case OFONO_GPRS_AUTH_METHOD_NONE:
+			snprintf(buf + len, sizeof(buf) - len - 3,
+					",\"%s\"", ctx->apn);
 			break;
 		}
+		break;
+	default:
+		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
+				ctx->apn);
+		break;
 	}
 
 	if (g_at_chat_send(gcd->chat, buf, none_prefix,
@@ -360,9 +359,12 @@ static void cgev_notify(GAtResult *result, gpointer user_data)
 	struct ofono_gprs_context *gc = user_data;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	const char *event;
-	int cid;
+	unsigned int cid;
+	int r;
 	GAtResultIter iter;
 
+	DBG("");
+
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CGEV:"))
@@ -371,18 +373,23 @@ static void cgev_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_unquoted_string(&iter, &event))
 		return;
 
-	if (g_str_has_prefix(event, "NW DEACT") == FALSE)
+	if (g_str_has_prefix(event, "NW PDN DEACT"))
+		r = sscanf(event, "%*s %*s %*s %u", &cid);
+	else if (g_str_has_prefix(event, "ME PDN DEACT"))
+		r = sscanf(event, "%*s %*s %*s %u", &cid);
+	else if (g_str_has_prefix(event, "NW DEACT"))
+		r = sscanf(event, "%*s %*s %u", &cid);
+	else if (g_str_has_prefix(event, "ME DEACT"))
+		r = sscanf(event, "%*s %*s %u", &cid);
+	else
 		return;
 
-	if (!g_at_result_iter_skip_next(&iter))
+	if (r != 1)
 		return;
 
-	if (!g_at_result_iter_next_number(&iter, &cid))
-		return;
+	DBG("cid %u", cid);
 
-	DBG("cid %d", cid);
-
-	if ((unsigned int) cid != gcd->active_context)
+	if (cid != gcd->active_context)
 		return;
 
 	if (gcd->state != STATE_IDLE && gcd->ppp)
@@ -411,9 +418,11 @@ static void at_cgdata_test_cb(gboolean ok, GAtResult *result,
 		goto error;
 	}
 
-	if (!g_at_result_iter_open_list(&iter)) {
-		DBG("no list found");
-		goto error;
+	if (gcd->vendor != OFONO_VENDOR_QUECTEL_SERIAL) {
+		if (!g_at_result_iter_open_list(&iter)) {
+			DBG("no list found");
+			goto error;
+		}
 	}
 
 	while (!found && g_at_result_iter_next_string(&iter, &data_type)) {
@@ -449,10 +458,6 @@ static int at_gprs_context_probe(struct ofono_gprs_context *gc,
 
 	ofono_gprs_context_set_data(gc, gcd);
 
-	chat = g_at_chat_get_slave(gcd->chat);
-	if (chat == NULL)
-		return 0;
-
 	switch (vendor) {
 	case OFONO_VENDOR_SIMCOM_SIM900:
 		gcd->use_atd99 = FALSE;
@@ -462,7 +467,10 @@ static int at_gprs_context_probe(struct ofono_gprs_context *gc,
 						at_cgdata_test_cb, gc, NULL);
 	}
 
-	g_at_chat_register(chat, "+CGEV:", cgev_notify, FALSE, gc, NULL);
+	chat = g_at_chat_get_slave(gcd->chat);
+	if (chat)
+		g_at_chat_register(chat, "+CGEV:", cgev_notify, FALSE, gc,
+									NULL);
 
 	return 0;
 }
@@ -494,7 +502,6 @@ static void at_gprs_context_remove(struct ofono_gprs_context *gc)
 }
 
 static const struct ofono_gprs_context_driver driver = {
-	.name			= "atmodem",
 	.probe			= at_gprs_context_probe,
 	.remove			= at_gprs_context_remove,
 	.activate_primary	= at_gprs_activate_primary,
@@ -502,12 +509,4 @@ static const struct ofono_gprs_context_driver driver = {
 	.detach_shutdown	= at_gprs_detach_shutdown,
 };
 
-void at_gprs_context_init(void)
-{
-	ofono_gprs_context_driver_register(&driver);
-}
-
-void at_gprs_context_exit(void)
-{
-	ofono_gprs_context_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs_context, atmodem, &driver)
