@@ -34,14 +34,14 @@
 #include <ofono/modem.h>
 #include <ofono/sim.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 #include "simutil.h"
 #include "vendor.h"
 #include "util.h"
 
-#include "atmodem.h"
-
 #define EF_STATUS_INVALIDATED 0
 #define EF_STATUS_VALID 1
 
@@ -107,6 +107,7 @@ static void get_response_common_cb(gboolean ok, GAtResult *result,
 	int str;
 	unsigned char access[3];
 	unsigned char file_status;
+	gboolean result_found = FALSE;
 
 	decode_at_error(&error, g_at_result_final_response(result));
 
@@ -117,11 +118,18 @@ static void get_response_common_cb(gboolean ok, GAtResult *result,
 
 	g_at_result_iter_init(&iter, result);
 
-	if (!g_at_result_iter_next(&iter, prefix))
-		goto error;
+	/* some Telit modems sometimes return incomplete result lines */
+	/* get first result line with mandatory parameters */
+	while (g_at_result_iter_next(&iter, prefix)) {
+		if (!g_at_result_iter_next_number(&iter, &sw1))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &sw2))
+			continue;
+		result_found = TRUE;
+	}
 
-	g_at_result_iter_next_number(&iter, &sw1);
-	g_at_result_iter_next_number(&iter, &sw2);
+	if (!result_found)
+		goto error;
 
 	if (!g_at_result_iter_next_hexstring(&iter, &response, &len) ||
 			(sw1 != 0x90 && sw1 != 0x91 && sw1 != 0x92) ||
@@ -199,6 +207,7 @@ static void at_sim_read_info(struct ofono_sim *sim, int fileid,
 	case OFONO_VENDOR_SPEEDUP:
 	case OFONO_VENDOR_QUALCOMM_MSM:
 	case OFONO_VENDOR_SIMCOM:
+	case OFONO_VENDOR_DROID:
 		/* Maximum possible length */
 		len += sprintf(buf + len, ",0,0,255");
 		break;
@@ -1198,6 +1207,7 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 			return;
 		break;
 	case OFONO_VENDOR_SIMCOM:
+	case OFONO_VENDOR_SIMCOM_A76XX:
 		if (g_at_chat_send(sd->chat, "AT+SPIC", simcom_spic_prefix,
 					simcom_spic_cb, cbd, g_free) > 0)
 			return;
@@ -1213,11 +1223,12 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 			return;
 		break;
 	case OFONO_VENDOR_QUECTEL:
+	case OFONO_VENDOR_QUECTEL_EC2X:
 		if (g_at_chat_send(sd->chat, "AT+QPINC?", qpinc_prefix,
 					at_qpinc_cb, cbd, g_free) > 0)
 			return;
 		break;
-	case OFONO_VENDOR_QUECTEL_M95:
+	case OFONO_VENDOR_QUECTEL_SERIAL:
 		if (g_at_chat_send(sd->chat, "AT+QTRPIN", qtrpin_prefix,
 					at_qtrpin_cb, cbd, g_free) > 0)
 			return;
@@ -1353,8 +1364,8 @@ static void at_pin_send_cb(gboolean ok, GAtResult *result,
 	case OFONO_VENDOR_ALCATEL:
 	case OFONO_VENDOR_HUAWEI:
 	case OFONO_VENDOR_SIMCOM:
+	case OFONO_VENDOR_SIMCOM_A76XX:
 	case OFONO_VENDOR_SIERRA:
-	case OFONO_VENDOR_QUECTEL_M95:
 		/*
 		 * On ZTE modems, after pin is entered, SIM state is checked
 		 * by polling CPIN as their modem doesn't provide unsolicited
@@ -1639,7 +1650,17 @@ static void at_discover_apps(struct ofono_sim *sim,
 				void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
-	struct cb_data *cbd = cb_data_new(cb, data);
+	struct cb_data *cbd;
+
+	/*
+	 * QUECTEL EC2X and SIMCom A7605E-H reboots when executing the AT+CUAD
+	 * command with SIM cards of some operators
+	 */
+	if (sd->vendor == OFONO_VENDOR_QUECTEL_EC2X
+			|| sd->vendor == OFONO_VENDOR_SIMCOM_A76XX)
+		goto error;
+
+	cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(sd->chat, "AT+CUAD", cuad_prefix,
 			at_discover_apps_cb, cbd, g_free) > 0)
@@ -1647,6 +1668,7 @@ static void at_discover_apps(struct ofono_sim *sim,
 
 	g_free(cbd);
 
+error:
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
@@ -2081,7 +2103,6 @@ static void at_set_active_card_slot(struct ofono_sim *sim, unsigned int index,
 }
 
 static const struct ofono_sim_driver driver = {
-	.name			= "atmodem",
 	.probe			= at_sim_probe,
 	.remove			= at_sim_remove,
 	.read_file_info		= at_sim_read_info,
@@ -2109,28 +2130,4 @@ static const struct ofono_sim_driver driver = {
 	.set_active_card_slot	= at_set_active_card_slot
 };
 
-static const struct ofono_sim_driver driver_noef = {
-	.name			= "atmodem-noef",
-	.probe			= at_sim_probe,
-	.remove			= at_sim_remove,
-	.read_imsi		= at_read_imsi,
-	.query_passwd_state	= at_pin_query,
-	.query_pin_retries	= at_pin_retries_query,
-	.send_passwd		= at_pin_send,
-	.reset_passwd		= at_pin_send_puk,
-	.lock			= at_pin_enable,
-	.change_passwd		= at_change_passwd,
-	.query_facility_lock	= at_query_clck,
-};
-
-void at_sim_init(void)
-{
-	ofono_sim_driver_register(&driver);
-	ofono_sim_driver_register(&driver_noef);
-}
-
-void at_sim_exit(void)
-{
-	ofono_sim_driver_unregister(&driver);
-	ofono_sim_driver_unregister(&driver_noef);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sim, atmodem, &driver)
