@@ -25,6 +25,7 @@
 
 #include <errno.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <glib.h>
 #include <gatchat.h>
@@ -47,6 +48,7 @@
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 
+#include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
 
 
@@ -69,50 +71,19 @@ static void wavecom_debug(const char *str, void *user_data)
 static int wavecom_enable(struct ofono_modem *modem)
 {
 	GAtChat *chat;
-	GIOChannel *channel;
-	GAtSyntax *syntax;
-	const char *device;
-	GHashTable *options;
 
 	DBG("%p", modem);
 
-	device = ofono_modem_get_string(modem, "Device");
-	if (device == NULL)
+	chat = at_util_open_device(modem, "Device", wavecom_debug, "",
+					"Baud", "115200",
+					"Parity", "none",
+					"StopBits", "1",
+					"DataBits", "8",
+					NULL);
+	if (!chat)
 		return -EINVAL;
 
-	options = g_hash_table_new(g_str_hash, g_str_equal);
-
-	if (options == NULL)
-		return -ENOMEM;
-
-	g_hash_table_insert(options, "Baud", "115200");
-	g_hash_table_insert(options, "Parity", "none");
-	g_hash_table_insert(options, "StopBits", "1");
-	g_hash_table_insert(options, "DataBits", "8");
-
-	channel = g_at_tty_open(device, options);
-
-	g_hash_table_destroy(options);
-
-	if (channel == NULL)
-		return -EIO;
-
-	/*
-	 * Could not figure out whether it is fully compliant or not, use
-	 * permissive for now
-	 * */
-	syntax = g_at_syntax_new_gsm_permissive();
-
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
-		return -ENOMEM;
-
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(chat, wavecom_debug, "");
-
+	g_at_chat_add_terminator(chat, "+CPIN:", 6, TRUE);
 	ofono_modem_set_data(modem, chat);
 
 	return 0;
@@ -134,28 +105,44 @@ static int wavecom_disable(struct ofono_modem *modem)
 static void wavecom_pre_sim(struct ofono_modem *modem)
 {
 	GAtChat *chat = ofono_modem_get_data(modem);
+	const char *model;
+	enum ofono_vendor vendor = 0;
+	struct ofono_sim *sim;
 
 	DBG("%p", modem);
 
+	model = ofono_modem_get_string(modem, "Model");
+	if (model && strcmp(model, "Q2XXX") == 0)
+		vendor = OFONO_VENDOR_WAVECOM_Q2XXX;
+
 	ofono_devinfo_create(modem, 0, "atmodem", chat);
-	ofono_sim_create(modem, OFONO_VENDOR_WAVECOM, "atmodem", chat);
+	sim = ofono_sim_create(modem, vendor, "atmodem", chat);
 	ofono_voicecall_create(modem, 0, "atmodem", chat);
+
+	if (vendor == OFONO_VENDOR_WAVECOM_Q2XXX)
+		ofono_sim_inserted_notify(sim, TRUE);
 }
 
 static void wavecom_post_sim(struct ofono_modem *modem)
 {
 	GAtChat *chat = ofono_modem_get_data(modem);
 	struct ofono_message_waiting *mw;
+	const char *model;
+	enum ofono_vendor vendor = 0;
 
 	DBG("%p", modem);
 
+	model = ofono_modem_get_string(modem, "Model");
+	if (model && strcmp(model, "Q2XXX") == 0)
+		vendor = OFONO_VENDOR_WAVECOM_Q2XXX;
+
 	ofono_ussd_create(modem, 0, "atmodem", chat);
 	ofono_call_forwarding_create(modem, 0, "atmodem", chat);
 	ofono_call_settings_create(modem, 0, "atmodem", chat);
 	ofono_netreg_create(modem, 0, "atmodem", chat);
 	ofono_call_meter_create(modem, 0, "atmodem", chat);
 	ofono_call_barring_create(modem, 0, "atmodem", chat);
-	ofono_sms_create(modem, 0, "atmodem", chat);
+	ofono_sms_create(modem, vendor, "atmodem", chat);
 	ofono_phonebook_create(modem, 0, "atmodem", chat);
 
 	mw = ofono_message_waiting_create(modem);
@@ -164,7 +151,6 @@ static void wavecom_post_sim(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver wavecom_driver = {
-	.name		= "wavecom",
 	.probe		= wavecom_probe,
 	.remove		= wavecom_remove,
 	.enable		= wavecom_enable,
@@ -173,15 +159,4 @@ static struct ofono_modem_driver wavecom_driver = {
 	.post_sim	= wavecom_post_sim,
 };
 
-static int wavecom_init(void)
-{
-	return ofono_modem_driver_register(&wavecom_driver);
-}
-
-static void wavecom_exit(void)
-{
-	ofono_modem_driver_unregister(&wavecom_driver);
-}
-
-OFONO_PLUGIN_DEFINE(wavecom, "Wavecom driver", VERSION,
-		OFONO_PLUGIN_PRIORITY_DEFAULT, wavecom_init, wavecom_exit)
+OFONO_MODEM_DRIVER_BUILTIN(wavecom, &wavecom_driver)
